<?php

namespace Drupal\coupons_migrate\Plugin\Commerce\PromotionOffer;

use Drupal\commerce_order\Adjustment;
use Drupal\commerce_promotion\Entity\PromotionInterface;
use Drupal\commerce_promotion\Plugin\Commerce\PromotionOffer\PromotionOfferBase;
use Drupal\commerce_price\Price;
use Drupal\Core\Entity\EntityInterface;

/**
 * Provides the Set Price for orders.
 *
 * @CommercePromotionOffer(
 *   id = "set_price",
 *   label = @Translation("Set Price"),
 *   entity_type = "commerce_order",
 * )
 */
class SetPrice extends PromotionOfferBase {

  use SetPriceTrait;

  /**
   * {@inheritdoc}
   */
  public function apply(EntityInterface $entity, PromotionInterface $promotion) {
    $this->assertEntity($entity);
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $entity;
    $subtotal_price = $order->getSubtotalPrice();

    if (!$subtotal_price || !$subtotal_price->isPositive()) {
      return;
    }
    $amount = $this->getAmount();
    $total_price = $order->getTotalPrice();

    // Extract numeric values from Price objects for calculation.
    $subtotal_number = $total_price->getNumber();

    $new_price_number = $amount->getNumber();

    // Calculate the difference between the current subtotal and the new price.
    $price_difference = $subtotal_number - $new_price_number;

    foreach ($order->getItems() as $order_item) {

      if ($amount->greaterThan($subtotal_price)) {
        // Convert the numeric adjustment back to a Price object.
        $adjustment_amount = new Price($new_price_number, $subtotal_price->getCurrencyCode());

      }
      else {
        $adjustment_amount = new Price('-' . $price_difference, $subtotal_price->getCurrencyCode());
      }

      // Add an adjustment to the order item for the proportional discount.
      $order_item->addAdjustment(new Adjustment([
        'type' => 'promotion',
        'label' => $promotion->getDisplayName() ?: $this->t('Discount'),
      // Discounts are negative amounts.
        'amount' => $adjustment_amount,
        'source_id' => $promotion->id(),
      ]));
    }

  }

}
